/*********************************************************************************
 * Android 2.3.3 (API 10) <-USB-> Arduino Mega ADK
 * Diese App stellt eine Verbindung zwischen einem Android Gert ( Android 2.3.3, API10) und einem Arduino MEGA ADK her. 
 * Zu Demonstrationszwecken ist die Steuerung eines GPIO Pins (LED) implementiert.
 *********************************************************************************/

package com.MSTUSBTS.AndroidArduino;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.ParcelFileDescriptor;
import android.view.View;
import android.widget.TextView;
import android.widget.ToggleButton;

import com.MSTUSBTS.AndroidArduino.R;
import com.android.future.usb.UsbAccessory;
import com.android.future.usb.UsbManager;

public class MainActivity extends Activity {

	//-------------------bentigte Elemente fr die Initialisierung der USB Verbindung--------
	private static final String ACTION_USB_PERMISSION = "com.android.example.USB_PERMISSION";
	private UsbManager usbManager;
	private PendingIntent pendingIntent;
	private boolean mPermissionRequestPending;
	UsbAccessory usbAccessory;
	ParcelFileDescriptor FileDescriptor;
	FileInputStream InputStream;
	FileOutputStream OutputStream;
	//-------------------Elemente der GUI-----------------------------------------------------
	private ToggleButton buttonLED;
	TextView refresh;
	TextView status;

	//-------------------Handling der eingehenden USB Daten-----------------------------------
	private final BroadcastReceiver usbReceiver = new BroadcastReceiver() {
    
		public void onReceive(Context context, Intent intent) {
			String action = intent.getAction();
			if (ACTION_USB_PERMISSION.equals(action)) {
				synchronized (this) {
					UsbAccessory accessory = UsbManager.getAccessory(intent);
					if (intent.getBooleanExtra(
							UsbManager.EXTRA_PERMISSION_GRANTED, false)) {
						openAccessory(accessory);
					} else {

					}
					mPermissionRequestPending = false;
				}
			} else if (UsbManager.ACTION_USB_ACCESSORY_DETACHED.equals(action)) {
				UsbAccessory accessory = UsbManager.getAccessory(intent);
				if (accessory != null && accessory.equals(usbAccessory)) {
					closeAccessory();
				}
			}
		}
	};

	//-------------------Verhalten bei Programmstart -> Initialisierung ----------------------
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		usbManager = UsbManager.getInstance(this);
		pendingIntent = PendingIntent.getBroadcast(this, 0, new Intent(
				ACTION_USB_PERMISSION), 0);
		IntentFilter filter = new IntentFilter(ACTION_USB_PERMISSION);
		filter.addAction(UsbManager.ACTION_USB_ACCESSORY_DETACHED);
		registerReceiver(usbReceiver, filter);

		if (getLastNonConfigurationInstance() != null) {
			usbAccessory = (UsbAccessory) getLastNonConfigurationInstance();
			openAccessory(usbAccessory);
		}

		setContentView(R.layout.main);
		buttonLED = (ToggleButton) findViewById(R.id.togglebutton);
		status = (TextView) findViewById(R.id.status);
		refresh = (TextView) findViewById(R.id.aktualisieren);
	}
	

	//-------------------Verhalten bei nderungen der Konfiguration---------------------------
	public Object onRetainNonConfigurationInstance() {
		if (usbAccessory != null) {
			return usbAccessory;
		} else {
			return super.onRetainNonConfigurationInstance();
		}
	}

	//-------------------Verhalten bei Wiederaufruf des Programmes----------------------------
	public void onResume() {
		super.onResume();

		if (InputStream != null && OutputStream != null) {
			return;
		}

		UsbAccessory[] accessories = usbManager.getAccessoryList();
		UsbAccessory accessory = (accessories == null ? null : accessories[0]);
		if (accessory != null) {
			if (usbManager.hasPermission(accessory)) {
				openAccessory(accessory);
			} else {
				synchronized (usbReceiver) {
					if (!mPermissionRequestPending) {
						usbManager.requestPermission(accessory, pendingIntent);
						mPermissionRequestPending = true;
					}
				}
			}
		} else {

		}
	}

	//-------------------ffnet die USB-Verbindung--------------------------------------------
	private void openAccessory(UsbAccessory accessory) {
		FileDescriptor = usbManager.openAccessory(accessory);
		if (FileDescriptor != null) {
			usbAccessory = accessory;
			FileDescriptor fd = FileDescriptor.getFileDescriptor();
			InputStream = new FileInputStream(fd);
			OutputStream = new FileOutputStream(fd);

		} else {

		}
	}
	//-------------------Schliet die USB-Verbindung------------------------------------------
	private void closeAccessory() {
		try {
			if (FileDescriptor != null) {
				FileDescriptor.close();
			}
		} catch (IOException e) {
		} finally {
			FileDescriptor = null;
			usbAccessory = null;
		}
	}

	//-------------------Funktion zum lesen des ToggleButtons---------------------------------
	//Diese Funktion liest den Wert des Buttons & gibt diesen Wert ber die USB Verbindung weiter an den Arduino.
	//Dort wird dieser Wert als Status der LED (Des GPIO Pins) interpretiert --> Die LED wird Ein/Aus geschaltet
	public void writegpio(View v) {

		byte[] buffer = new byte[1];
	
		
		
		if (buttonLED.isChecked())
			buffer[0] = (byte) 1; // Nachricht 1 -> LED einschalten.

		else
			buffer[0] = (byte) 0; // Nachricht 0 -> LED auschalten.

		if (OutputStream != null) {
			try {
				OutputStream.write(buffer); // Nachricht wird in den OutputStream geschrieben.
			} catch (IOException e) {

			}

		}
	}


	/*Android sendet Nachricht 2 zum Arduino, um digitale Input auszulesen. */
	// Diese Funktion sendet eine Anfrage an den Arduino und bekommt als Antwort den Status der LED (des GPIO Pins)
	public void gpioread(View view) {
		
		byte[] buffer = new byte[1];
		
		buffer[0] = (byte) 2; 				//Die 2 wird an den Arduino gesendet
		if (OutputStream != null) {			
			try {
				OutputStream.write(buffer); 
			} catch (IOException e) {

			}
		}	

		try {								//Die Antwort des Arduino wird gelesen
			InputStream.read(buffer);

		} catch (IOException e) {

		}

		
		if (buffer[0] == (byte) 1) {		//Die gelesene Antwort wird interpretiert & angezeigt

			status.setText("HIGH");
		}else{
			status.setText("LOW");
		}

		

	}

}
