package com.tuhh.mikrosystem.handysensoren;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;

public class SensorActivity_Beschleunigung extends ActionBarActivity implements SensorEventListener
{
	// SensorManager, Sensorvariablen, Wertearrays, Schwellen und Steuerelementvariablen anlegen
    private SensorManager
			BeschleunigungsSensorManager = null;

	private Sensor
			sensor_Beschleunigung_g = null, sensor_Beschleunigung_lin = null;

	private float[]
			Sensorwerte_neu_g = null, Sensorwerte_neu_lin = null,
			Sensorwerte_alt_g = {0,0,0}, Sensorwerte_alt_lin = {0,0,0},
			Sensorwerte_differenz_g = {0,0,0}, Sensorwerte_differenz_lin = {0,0,0};

	private TextView
			x_g_TextView = null, y_g_TextView = null, z_g_TextView = null,
			x_lin_TextView = null, y_lin_TextView = null, z_lin_TextView = null;

	private float SENSORWERTE_SCHWELLE_g = (float)0.6;                                              // Schwelle zum vorherigen Wert um Erschütterungen festzustellen und rot zu markieren
	private float SENSORWERTE_SCHWELLE_lin = (float)0.3;                                            // Schwelle zum vorherigen Wert um Erschütterungen festzustellen und rot zu markieren


	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_sensor_beschleunigung);                                    // Layout starten

		BeschleunigungsSensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);    // Beschleunigungssensorabfrage im System registrieren
		LayoutInit();                                                                               // Layout initialisieren

		Intent intent = getIntent();                                                                // Aus der Übergabe aus dem letzten Layout schauen welche Sensoren verfügbar sind
		String message = intent.getStringExtra(MenueActivity.EXTRA_MESSAGE);

		switch(Integer.valueOf(message))                                                            // Gucken welche Sensoren verbaut sind (wurde übergeben aus dem Menü)
		{
			case 0: break;                                                                          // 0 heißt beide sind verbaut
			case 1:
				TextView Beschleunigung_lin = (TextView)findViewById(R.id.sensor_beschleunigung_lin); // Heißt, dass nur der normale Sensor verbaut ist
				Beschleunigung_lin.setVisibility(TextView.INVISIBLE);                               // Schaltflächen und Werte für den linearen verbergen
				x_lin_TextView.setVisibility(TextView.INVISIBLE);
				y_lin_TextView.setVisibility(TextView.INVISIBLE);
				z_lin_TextView.setVisibility(TextView.INVISIBLE);
				break;
			case 2:
				TextView Beschleunigung_g = (TextView)findViewById(R.id.sensor_beschleunigung_g); // Heißt, dass nur der lineare Sensor verbaut ist
				Beschleunigung_g.setVisibility(TextView.INVISIBLE);                                 // Schaltflächen und Werte für den normalen verbergen
				x_g_TextView.setVisibility(TextView.INVISIBLE);
				y_g_TextView.setVisibility(TextView.INVISIBLE);
				z_g_TextView.setVisibility(TextView.INVISIBLE);
				break;
		}
	}

	private void LayoutInit()
	{
		sensor_Beschleunigung_g = BeschleunigungsSensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);     // Sensor im System registrieren
		sensor_Beschleunigung_lin = BeschleunigungsSensorManager.getDefaultSensor(Sensor.TYPE_LINEAR_ACCELERATION);     // Sensor im System registrieren

		x_g_TextView = (TextView) this.findViewById(R.id.sensor_beschleunigung_g_x);                // Textfelder in Variablen legen
		y_g_TextView = (TextView) this.findViewById(R.id.sensor_beschleunigung_g_y);
		z_g_TextView = (TextView) this.findViewById(R.id.sensor_beschleunigung_g_z);
		x_lin_TextView = (TextView) this.findViewById(R.id.sensor_beschleunigung_lin_x);
		y_lin_TextView = (TextView) this.findViewById(R.id.sensor_beschleunigung_lin_y);
		z_lin_TextView = (TextView) this.findViewById(R.id.sensor_beschleunigung_lin_z);
	}

	public void onSensorChanged(SensorEvent event)                                                  // Wenn der SensorManager ein neues Event bemerkt
	{
		if(event.sensor == sensor_Beschleunigung_g)                                                 // War es der normale Beschleunigungssensor?
		{
			Sensorwerte_neu_g = event.values.clone();                                               // Werte aus dem Event auslesen

			for(int i=0; i<=2; i++)                                                                 // Differenz zum alten Wert bilden
			{
				Sensorwerte_differenz_g[i] = Sensorwerte_alt_g[i] - Sensorwerte_neu_g[i];           // Differenz ausrechnen
				if(Sensorwerte_differenz_g[i] < 0){Sensorwerte_differenz_g[i]=Sensorwerte_differenz_g[i]*(-1);}     // Betrag bilden
			}
			if(Sensorwerte_differenz_g[0] > SENSORWERTE_SCHWELLE_g){x_g_TextView.setTextColor(Color.RED); x_lin_TextView.setTextColor(Color.RED);} // Wenn eine Erschütterung, also eine große Abweichung zum vorherigen Wert festgestellt wird -> in rot markieren
			else{x_g_TextView.setTextColor(Color.WHITE); x_lin_TextView.setTextColor(Color.WHITE);}     // Wenn nicht -> Weiß lassen, wie sonst auch
			if(Sensorwerte_differenz_g[1] > SENSORWERTE_SCHWELLE_g){y_g_TextView.setTextColor(Color.RED); y_lin_TextView.setTextColor(Color.RED);}
			else{y_g_TextView.setTextColor(Color.WHITE); y_lin_TextView.setTextColor(Color.WHITE);}
			if(Sensorwerte_differenz_g[2] > SENSORWERTE_SCHWELLE_g){z_g_TextView.setTextColor(Color.RED); z_lin_TextView.setTextColor(Color.RED);}
			else{z_g_TextView.setTextColor(Color.WHITE); z_lin_TextView.setTextColor(Color.WHITE);}

			x_g_TextView.setText(getString(R.string.sensor_beschleunigung_in_x) + " " + String.valueOf(FloatRunden((-1)*Sensorwerte_neu_g[0],2)) + " " + getString(R.string.sensor_beschleunigung_einheit));    // Wert eintragen
			y_g_TextView.setText(getString(R.string.sensor_beschleunigung_in_y) + " " + String.valueOf(FloatRunden((-1)*Sensorwerte_neu_g[1],2)) + " " + getString(R.string.sensor_beschleunigung_einheit));
			z_g_TextView.setText(getString(R.string.sensor_beschleunigung_in_z) + " " + String.valueOf(FloatRunden((-1)*Sensorwerte_neu_g[2],2)) + " " + getString(R.string.sensor_beschleunigung_einheit));

			Sensorwerte_alt_g = Sensorwerte_neu_g;                                                  // neuen Sensorwerte fürs nächste Mal speichern
		}
		if(event.sensor == sensor_Beschleunigung_lin)                                               // War es der lineare Beschleunigungssensor
		{
			Sensorwerte_neu_lin = event.values.clone();                                             // Werte aus dem Event auslesen

			for(int i=0; i<=2; i++)                                                                 // Differenz zum alten Wert bilden
			{
				Sensorwerte_differenz_lin[i] = Sensorwerte_alt_lin[i] - Sensorwerte_neu_lin[i];           // Differenz ausrechnen
				if(Sensorwerte_differenz_lin[i] < 0){Sensorwerte_differenz_lin[i]=Sensorwerte_differenz_lin[i]*(-1);}     // Betrag bilden
			}
			if(Sensorwerte_differenz_lin[0] > SENSORWERTE_SCHWELLE_lin){x_lin_TextView.setTextColor(Color.RED); x_g_TextView.setTextColor(Color.RED);} // Wenn eine Erschütterung, also eine große Abweichung zum vorherigen Wert festgestellt wird -> in rot markieren
			else{x_lin_TextView.setTextColor(Color.WHITE); x_g_TextView.setTextColor(Color.WHITE);}     // Wenn nicht -> Weiß lassen, wie sonst auch
			if(Sensorwerte_differenz_lin[1] > SENSORWERTE_SCHWELLE_lin){y_lin_TextView.setTextColor(Color.RED); y_g_TextView.setTextColor(Color.RED);}
			else{y_lin_TextView.setTextColor(Color.WHITE); y_g_TextView.setTextColor(Color.WHITE);}
			if(Sensorwerte_differenz_lin[2] > SENSORWERTE_SCHWELLE_lin){z_lin_TextView.setTextColor(Color.RED); z_g_TextView.setTextColor(Color.RED);}
			else{z_lin_TextView.setTextColor(Color.WHITE); z_g_TextView.setTextColor(Color.WHITE);}

			x_lin_TextView.setText(getString(R.string.sensor_beschleunigung_in_x) + " " + String.valueOf(FloatRunden((-1)*Sensorwerte_neu_lin[0],2)) + " " + getString(R.string.sensor_beschleunigung_einheit));    // Wert eintragen
			y_lin_TextView.setText(getString(R.string.sensor_beschleunigung_in_y) + " " + String.valueOf(FloatRunden((-1)*Sensorwerte_neu_lin[1],2)) + " " + getString(R.string.sensor_beschleunigung_einheit));
			z_lin_TextView.setText(getString(R.string.sensor_beschleunigung_in_z) + " " + String.valueOf(FloatRunden((-1)*Sensorwerte_neu_lin[2],2)) + " " + getString(R.string.sensor_beschleunigung_einheit));

			Sensorwerte_alt_lin = Sensorwerte_neu_lin;                                              // neuen Sensorwerte fürs nächste Mal speichern
		}
	}

	private double FloatRunden(double wert, int stellen)                                            // Werte auf "stellen"- Stellen runden
	{
		double gerundet = Math.round(wert * Math.pow(10d, stellen));
		return gerundet / Math.pow(10d, stellen);
	}

	protected void onPause()                                                                        // Wenn die App minimiert wird
	{
		super.onPause();
		BeschleunigungsSensorManager.unregisterListener(this);                                      // alle Sensoren derigistrieren, sonst gibt es weiterhin Events
	}

	protected void onResume()                                                                       // Wenn die App wieder geöffnet wird
	{
		super.onResume();
		BeschleunigungsSensorManager.registerListener(this, sensor_Beschleunigung_g,SensorManager.SENSOR_DELAY_NORMAL);     // Sensor wieder im System eintragen, dann können Events kommen
		BeschleunigungsSensorManager.registerListener(this, sensor_Beschleunigung_lin, SensorManager.SENSOR_DELAY_NORMAL);  // ||
	}

	public void onAccuracyChanged(Sensor sensor, int accuracy) {}

	public boolean onCreateOptionsMenu(Menu menu)           // Wieder Menü anlegen
	{
		getMenuInflater().inflate(R.menu.menu_klein, menu);
		return true;
	}

	public boolean onOptionsItemSelected(MenuItem item)
	{
		switch (item.getItemId())
		{
			case R.id.action_info:
				Info();
				return true;

			default:
				return super.onOptionsItemSelected(item);
		}
	}

	private void Info()
	{
		AlertDialog alertDialog = new AlertDialog.Builder(this).create();
		alertDialog.setTitle("INFO");
		alertDialog.setMessage(getString(R.string.menue_info_text));
		alertDialog.setButton("OK", new DialogInterface.OnClickListener()
		{public void onClick(DialogInterface dialog, int which) {}});
		alertDialog.show();
	}
}
