package com.tuhh.mikrosystem.handysensoren;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.widget.LinearLayout;

public class SensorActivity_Bildschirm extends ActionBarActivity
{
    private MultiTouchView mMultiTouchView = null;                                                  // Eine Art "Sensorvariable" anlegen

    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_sensor_bildschirm);                                        // Layout starten

        mMultiTouchView = new MultiTouchView(this);                                                 // Bildschirmvariable füllen

        LinearLayout layout = (LinearLayout) this.findViewById(R.id.layout_sensor_bildschirm_view);
        layout.addView(mMultiTouchView);                                                            // Den eingebettenen View starten
    }

    private class MultiTouchView extends View                                                       // View in dem dargestellt wird. "Ein Fenster in einem Fenster"
    {
        private Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);                                     // Ein Zeichenfenster anlegen

        final int MAX_NUMBER_OF_POINT = 10;                                                         // Maximal 10 Punkte gleichzeitig erfassen
        float[] x = new float[MAX_NUMBER_OF_POINT];                                                 // Koordinatenarray
        float[] y = new float[MAX_NUMBER_OF_POINT];                                                 // Koordinatenarray
        boolean[] touching = new boolean[MAX_NUMBER_OF_POINT];                                      // Wieviele gedrückt wurden

        public MultiTouchView(Context context, AttributeSet attrs, int defStyle)
        {
            super(context, attrs, defStyle);
            init();
        }

        public MultiTouchView(Context context, AttributeSet attrs)
        {
            super(context, attrs);
            init();
        }

        public MultiTouchView(Context context)
        {
            super(context);
            init();
        }

        void init()
        {
            paint.setStyle(Paint.Style.STROKE);
            paint.setStrokeWidth(1);
            paint.setColor(Color.WHITE);
        }

        protected void onDraw(Canvas canvas)
        {
            paint.setStyle(Paint.Style.FILL);                                                       // Alle Kreise ausfüllen
            for(int i = 0; i < MAX_NUMBER_OF_POINT; i++)                                            // Alle Kreise malen
            {
                if(touching[i])                                                                     // Wenn so viele noch gemalt worden sind, dann steht im Array eine 1 drin
                {
                    canvas.drawCircle(x[i], y[i], 50f, paint);                                      // Kreis malen wo gedrückt wurde
                }
            }
        }

        protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec)
        {
            setMeasuredDimension(MeasureSpec.getSize(widthMeasureSpec),MeasureSpec.getSize(heightMeasureSpec));     // Displaygröße bestimmen
        }

        public boolean onTouchEvent(MotionEvent event)
        {
            int action = (event.getAction() & MotionEvent.ACTION_MASK);                             // Wenn ein Druck registriert wurde
            int pointCount = event.getPointerCount();                                               // Daten des Drucks auslesen

            for (int i = 0; i < pointCount; i++)                                                    // Soviele Drücke wie registriert wurden abfragen
            {
                int id = event.getPointerId(i);                                                     // Nummer des Drucks auslesen

                if(id < MAX_NUMBER_OF_POINT)                                                        // Nur maximal 10 Drücke mitnehmen
                {
                    x[id] = (int)event.getX(i);                                                     // Koordinaten auslesen
                    y[id] = (int)event.getY(i);                                                     //     ||

                    if((action == MotionEvent.ACTION_DOWN)||(action == MotionEvent.ACTION_POINTER_DOWN)||(action == MotionEvent.ACTION_MOVE))
                    {
                        touching[id] = true;                                                        // Wenn Gedrückt oder bewegt wurde, wurde etwas berührt. Z.B. nicht wenn losgelassen wird
                    }
                    else
                    {
                        touching[id] = false;                                                       // Sonst war das was anderes und wird nicht behandelt
                    }
                }
            }

            invalidate();                                                                           // Bildschirm neu zeichnen lassen
            return true;
        }
    }

    public boolean onCreateOptionsMenu(Menu menu)                                                   // Menü erstellen
    {
        getMenuInflater().inflate(R.menu.menu_klein, menu);
        return true;
    }

    public boolean onOptionsItemSelected(MenuItem item)
    {
        switch (item.getItemId())
        {
            case R.id.action_info:
                Info();
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private void Info()
    {
        AlertDialog alertDialog = new AlertDialog.Builder(this).create();
        alertDialog.setTitle("INFO");
        alertDialog.setMessage(getString(R.string.menue_info_text));
        alertDialog.setButton("OK", new DialogInterface.OnClickListener()
        {public void onClick(DialogInterface dialog, int which) {}});
        alertDialog.show();
    }
}