package com.tuhh.mikrosystem.handysensoren;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;

public class SensorActivity_GPS extends ActionBarActivity
{
	// Steuerelementvariablen anlegen
    TextView
			sensor_gps_laengengrad_text = null, sensor_gps_breitengrad_text = null,
			sensor_gps_genauigkeit_text = null, sensor_gps_hoehe_text = null,
			sensor_gps_winkel_text = null, sensor_gps_geschwindigkeit_text = null;

	LocationManager GPSManager;     // GPS-Manager wird benötigt

	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_sensor_gps);                                               // Layout starten

        // Steuerelemente in Variablen legen
		sensor_gps_laengengrad_text = (TextView)this.findViewById(R.id.sensor_gps_text_laengengrad);
		sensor_gps_breitengrad_text = (TextView)this.findViewById(R.id.sensor_gps_text_breitengrad);
		sensor_gps_genauigkeit_text = (TextView)this.findViewById(R.id.sensor_gps_text_genauigkeit);
		sensor_gps_hoehe_text = (TextView)this.findViewById(R.id.sensor_gps_text_hoehe);
		sensor_gps_winkel_text = (TextView)this.findViewById(R.id.sensor_gps_text_winkel);
		sensor_gps_geschwindigkeit_text = (TextView)this.findViewById(R.id.sensor_gps_text_geschwindigkeit);

		StartGPSManager();                                                                          // GPS Manager starten
	}

	private void StartGPSManager()
	{
		GPSManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);                   // Service starten
		GPSManager.requestLocationUpdates("gps", 10000, 1, locationListener);                       // Standortevents registrieren -> alle 10secs
		Location Position = GPSManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);          // Letzte bekannte Position, falls möglich, auslesen
		if (Position != null) {Speichern(Position);}                                                // Position speichern und ausgeben
	}

    void Speichern(Location AktuellePosition)
	{
		// Alle möglichen Werte auslesen und im Layout anzeigen
        sensor_gps_laengengrad_text.setText(getString(R.string.sensor_gps_laengengrad) + " " + String.valueOf(FloatRunden(AktuellePosition.getLatitude(),3)) + " " + getString(R.string.sensor_gps_grad_einheit));
		sensor_gps_breitengrad_text.setText(getString(R.string.sensor_gps_breitengrad) + " " + String.valueOf(FloatRunden(AktuellePosition.getLongitude(),3)) + " " + getString(R.string.sensor_gps_grad_einheit));

		if(AktuellePosition.getAccuracy() == 0.0){sensor_gps_genauigkeit_text.setText(getString(R.string.sensor_gps_genauigkeit) + " " + getString(R.string.sensor_gps_nicht_verfuegbar));}
		else{sensor_gps_genauigkeit_text.setText(getString(R.string.sensor_gps_genauigkeit) + " " + String.valueOf(FloatRunden(AktuellePosition.getAccuracy(),3)) + " " + getString(R.string.sensor_gps_meter_einheit));}
		if(AktuellePosition.getAltitude() == 0.0){sensor_gps_hoehe_text.setText(getString(R.string.sensor_gps_hoehe) + " " + getString(R.string.sensor_gps_nicht_verfuegbar));}
		else{sensor_gps_hoehe_text.setText(getString(R.string.sensor_gps_hoehe) + " " + String.valueOf(FloatRunden(AktuellePosition.getAltitude(),3)) + " " + getString(R.string.sensor_gps_meter_einheit));}
		if(AktuellePosition.getBearing() == 0.0){sensor_gps_winkel_text.setText(getString(R.string.sensor_gps_winkel) + " " + getString(R.string.sensor_gps_nicht_verfuegbar));}
		else{sensor_gps_winkel_text.setText(getString(R.string.sensor_gps_winkel) + " " + String.valueOf(FloatRunden(AktuellePosition.getBearing(),3)) + " " + getString(R.string.sensor_gps_grad_einheit));}
		if(AktuellePosition.getSpeed() == 0.0){sensor_gps_geschwindigkeit_text.setText(getString(R.string.sensor_gps_geschwindigkeit) + " " + getString(R.string.sensor_gps_nicht_verfuegbar));}
		else{sensor_gps_geschwindigkeit_text.setText(getString(R.string.sensor_gps_geschwindigkeit) + " " + String.valueOf(FloatRunden(AktuellePosition.getSpeed(),3)) + " " + getString(R.string.sensor_gps_geschwindigkeit_einheit));}
    }

	private double FloatRunden(double wert, int stellen)                                            // Funktion zum Runden von Werten
	{
		double gerundet = Math.round(wert * Math.pow(10d, stellen));
		return gerundet / Math.pow(10d, stellen);
	}

   private final LocationListener locationListener = new LocationListener()                         // LocationListener wird gebraucht zur Standortabfrage -> dieser bekommt die Events
   {
        public void onLocationChanged(Location AktuellePosition)
		{
			Speichern(AktuellePosition);                                                            // Wenn sich die Position ändert -> Posi speichern
		}
        public void onProviderDisabled(String provider){}
        public void onProviderEnabled(String provider) {}
        public void onStatusChanged(String provider, int status, Bundle extras) {}
  };

	protected void onPause()                                                                        // Wenn die App minimiert wird
	{
		super.onPause();
		GPSManager.removeUpdates(locationListener);                                                 // GPS und Standortabfragen beenden -> energiesparender
	}

	protected void onResume()
	{
		super.onResume();
		StartGPSManager();                                                                          // Wenn die App wieder geöffnet wird -> GPS Manager wieder starten
	}

	public boolean onCreateOptionsMenu(Menu menu)                                                   // Menü anlegen
	{
		getMenuInflater().inflate(R.menu.menu_klein, menu);
		return true;
	}

	public boolean onOptionsItemSelected(MenuItem item)
	{
		switch (item.getItemId())
		{
			case R.id.action_info:
				Info();
				return true;

			default:
				return super.onOptionsItemSelected(item);
		}
	}

	private void Info()
	{
		AlertDialog alertDialog = new AlertDialog.Builder(this).create();
		alertDialog.setTitle("INFO");
		alertDialog.setMessage(getString(R.string.menue_info_text));
		alertDialog.setButton("OK", new DialogInterface.OnClickListener()
		{public void onClick(DialogInterface dialog, int which) {}});
		alertDialog.show();
	}
}