package com.tuhh.mikrosystem.handysensoren;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.support.v7.app.ActionBarActivity;
import android.util.DisplayMetrics;
import android.view.Display;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;

public class SensorActivity_Orientierung extends ActionBarActivity implements SensorEventListener
{
	// Variablen anlegen

    private SensorManager
			OrientierungsSensorManager = null;

	private Sensor
			sensor_Orientierung = null;

	private float[]
			Sensorwerte = null;

	private TextView
			xTextView  = null, yTextView = null, zTextView = null;

	private WasserwaageView
			mWasserwaageView = null;

	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_sensor_orientierung);                                      // Layout starten

		OrientierungsSensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);      // SensorManager anlegen
		sensor_Orientierung = OrientierungsSensorManager.getDefaultSensor(Sensor.TYPE_ORIENTATION); // Orientierungssensor im System registrieren

		xTextView = (TextView) this.findViewById(R.id.sensor_orientierung_x_wert);                  // Textfelder in Variablen legen
		yTextView = (TextView) this.findViewById(R.id.sensor_orientierung_y_wert);
		zTextView = (TextView) this.findViewById(R.id.sensor_orientierung_z_wert);

		mWasserwaageView = new WasserwaageView(this);                                               // Wieder ein "Fenster im Fenster" also ein neuer View wird angezeigt -> für die Wasserwaage

		LinearLayout layout = (LinearLayout) this.findViewById(R.id.layout_sensor_orientierung);    // In ein Lineares Layout einfügen
        layout.addView(mWasserwaageView);                                                           // Wasserwaagenview einfügen
	}

	public void onSensorChanged(SensorEvent event)                                                  // Wenn eine Sensoränderung detektiert wird
	{
		if(event.sensor == sensor_Orientierung)                                                     // Wenn es der Orientierungssensor war
		{
			Sensorwerte = event.values.clone();

			xTextView.setText(String.valueOf(Math.round(Sensorwerte[2]))+ " " +getString(R.string.sensor_orientierung_grad_einheit));                // Werte ausgeben
			yTextView.setText(String.valueOf(Math.round(Sensorwerte[1]))+ " " +getString(R.string.sensor_orientierung_grad_einheit));
			zTextView.setText(String.valueOf(Math.round(Sensorwerte[0]))+ " " +getString(R.string.sensor_orientierung_grad_einheit));
			mWasserwaageView.update(Sensorwerte[1], Sensorwerte[2]);                                // Wasserwaage refreshen
		}
	}

	private class WasserwaageView extends View                                                      // Wasserwaage zeichnen
	{
		private float wert1;
		private float wert2;
		private int RECHTECK_LAENGE  = 280;
		private int RECHTECK_MITTE = 50;
		private final int RECHTECK_BREITE  = 50;
		private final int KREIS_RADIUS     = 23;
		private final int RECHTECK_ABSTAND = 20; // Abstand Rand und zwischen Rechtecken
		private float EINHEIT = RECHTECK_LAENGE / 210.0f;  // Pixel pro Grad
		private Paint rechteckPaint  = null, kreisPaint  = null;

		public WasserwaageView(Context context)
		{
			super(context);

			rechteckPaint = new Paint();                                                            // Rechtecke konfigurieren
			rechteckPaint.setColor(Color.YELLOW);                                                   // Rechtecke sind gelb
			rechteckPaint.setStyle(Paint.Style.FILL);                                               // Ausfüllen nicht nur Konturen

			kreisPaint = new Paint();                                                               // Kreise konfigurieren (die Blase)
			kreisPaint.setColor(Color.RED);                                                         // Kreise sind Rot
			kreisPaint.setStyle(Paint.Style.FILL);                                                  // Auch Ausfüllen

			// Umrechung für verschiedene Displaygrößen
			Display display = getWindowManager().getDefaultDisplay();                               // Bildschirm abfragen
			DisplayMetrics outMetrics = new DisplayMetrics ();
			display.getMetrics(outMetrics);                                                         // Bildschirmgröße abfragen

			float density  = getResources().getDisplayMetrics().density;                            // Pixeldichte auslesen
			float dpHeight = outMetrics.heightPixels / density;                                     // Pixel in Höhe auslesen
			float dpWidth  = outMetrics.widthPixels / density;                                      // Pixel in Breite auslesen

			int HoeheInDP = RECHTECK_ABSTAND+350;                                                   // Die Wasserwaage soll mittig unter der Schrift sitzen und den Rest des Bildschirms ausfüllen
			int HoeheMaximal = Math.round(dpHeight)-HoeheInDP-RECHTECK_ABSTAND;                     // Maximale Höhe der Wasserwaage
			int BreiteMaximal = Math.round(dpWidth)- 2*RECHTECK_ABSTAND-RECHTECK_BREITE-RECHTECK_ABSTAND;   // Maximale Breite der Wasserwaage
			if((RECHTECK_LAENGE<BreiteMaximal) && (RECHTECK_LAENGE < HoeheMaximal)){}               // Wenn das mit der maximalen Breite passt -> alles okay
			else if(BreiteMaximal < HoeheMaximal){RECHTECK_LAENGE = BreiteMaximal; EINHEIT = RECHTECK_LAENGE / 270.0f;}      // Wenn nicht: Die Rechteckbreite maximal setzen
			else if(HoeheMaximal < BreiteMaximal){RECHTECK_LAENGE = HoeheMaximal; EINHEIT = RECHTECK_LAENGE / 270.0f;}      // Wenn nicht: Rechtecklänge maximal setzen

			RECHTECK_MITTE = Math.round(dpWidth/2)-(RECHTECK_BREITE+RECHTECK_ABSTAND+RECHTECK_LAENGE)/2;        // Mittelpunkt der ganzen Wasserwaage finden
			if(RECHTECK_MITTE < 16){RECHTECK_MITTE = 16;} //dimen                                   // 16pt dimen-Rahmen setzen
		}

		protected void onDraw(Canvas canvas)
		{
			float wert_y = wert1 * EINHEIT; // Umrechnung auf Pixel
			float wert_x = wert2 * EINHEIT;

			// Rechteck für y-Richtung (vorwärts/rückwärts-Neigung)
			Rect r = new Rect(RECHTECK_MITTE,RECHTECK_ABSTAND,RECHTECK_MITTE + RECHTECK_BREITE,RECHTECK_LAENGE+RECHTECK_ABSTAND);
			canvas.drawRect(r, rechteckPaint);

			// Rechteck für x-Richtung (links-rechts-Neigung)
			int rx = RECHTECK_MITTE + RECHTECK_BREITE + RECHTECK_ABSTAND;
			int ry = RECHTECK_ABSTAND + RECHTECK_LAENGE / 2 - RECHTECK_BREITE / 2;
			r = new Rect(rx, ry, rx + RECHTECK_LAENGE, ry + RECHTECK_BREITE);
			canvas.drawRect(r, rechteckPaint);

			// Null-Position für Kugel y-Richtung
			int nullPos1_x = RECHTECK_MITTE + RECHTECK_BREITE / 2;
			int nullPos1_y = RECHTECK_ABSTAND + RECHTECK_LAENGE / 2;

			// Null-Position für  Kugel x-Richtung
			int nullPos2_x = nullPos1_x + RECHTECK_BREITE / 2 + RECHTECK_ABSTAND + RECHTECK_LAENGE / 2;
			int nullPos2_y = nullPos1_y;

            // Nullposition hinmalen
            kreisPaint.setColor(Color.BLACK);
            kreisPaint.setStyle(Paint.Style.STROKE);
            canvas.drawCircle(nullPos1_x, nullPos1_y, KREIS_RADIUS, kreisPaint);
            canvas.drawCircle(nullPos2_x, nullPos2_y, KREIS_RADIUS, kreisPaint);
            kreisPaint.setColor(Color.RED);
            kreisPaint.setStyle(Paint.Style.FILL);

			// aktuelle Neigung hinzufügen
			nullPos1_y  += wert_y;
			nullPos2_x  += wert_x;

			// Sicherstellen, dass Rechtecke nicht verlassen werden
			int pos1_minY = 0 + KREIS_RADIUS+RECHTECK_ABSTAND;
			int pos1_maxY = RECHTECK_LAENGE - KREIS_RADIUS+RECHTECK_ABSTAND;
			nullPos1_y = (nullPos1_y < pos1_minY ? pos1_minY : nullPos1_y);
			nullPos1_y = (nullPos1_y > pos1_maxY ? pos1_maxY : nullPos1_y);

			int pos2_minX = RECHTECK_MITTE + RECHTECK_BREITE + RECHTECK_ABSTAND + KREIS_RADIUS;
			int pos2_maxX = RECHTECK_MITTE + RECHTECK_BREITE + RECHTECK_ABSTAND +  RECHTECK_LAENGE - KREIS_RADIUS;
			nullPos2_x = (nullPos2_x < pos2_minX ? pos2_minX : nullPos2_x);
			nullPos2_x = (nullPos2_x > pos2_maxX ? pos2_maxX : nullPos2_x);

			// Kreise zeichnen
			canvas.drawCircle(nullPos1_x, nullPos1_y, KREIS_RADIUS, kreisPaint);
			canvas.drawCircle(nullPos2_x, nullPos2_y, KREIS_RADIUS, kreisPaint);
		}

		public void update(float wert1, float wert2)                                                // Wenn ein neuer Wert gemessen wurde -> Wasserwaage neuzeichnen
		{
			this.wert1 = wert1;                                                                     // Werte in View kopieren
			this.wert2 = wert2;                                                                     // Koordinaten  ||
			this.invalidate();                                                                      // neu zeichnen lassen
		}
	}

	protected void onPause()                                                                        // Wenn die App oder das Layout minimiert werden
	{
		super.onPause();
		OrientierungsSensorManager.unregisterListener(this);                                        // Orientierungssensor aus den Registrierungen entfernen -> sonst gibt es immernoch Events
	}

	protected void onResume()                                                                       // Wenn die App oder das Layout wieder maximiert werden
	{
		super.onResume();
		OrientierungsSensorManager.registerListener(this, sensor_Orientierung,SensorManager.SENSOR_DELAY_GAME);     // Sensormanager wieder im System registrieren und auf erhöhte Empfindlichkeit schalten -> läuft dann flüssiger
	}

	public void onAccuracyChanged(Sensor sensor, int accuracy) {}

	public boolean onCreateOptionsMenu(Menu menu)                                                   // Wieder Menü anlegen
	{
		getMenuInflater().inflate(R.menu.menu_klein, menu);
		return true;
	}

	public boolean onOptionsItemSelected(MenuItem item)
	{
		switch (item.getItemId())
		{
			case R.id.action_info:
				Info();
				return true;

			default:
				return super.onOptionsItemSelected(item);
		}
	}

	private void Info()
	{
		AlertDialog alertDialog = new AlertDialog.Builder(this).create();
		alertDialog.setTitle("INFO");
		alertDialog.setMessage(getString(R.string.menue_info_text));
		alertDialog.setButton("OK", new DialogInterface.OnClickListener()
		{public void onClick(DialogInterface dialog, int which) {}});
		alertDialog.show();
	}
}
