package com.tuhh.mikrosystem.handysensoren;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.os.Handler;
import android.support.v7.app.ActionBarActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.ImageView;
import android.widget.TextView;

public class SensorActivity_Schritt extends ActionBarActivity implements SensorEventListener
{
	// Variablen anlegen

    private SensorManager
			SchrittSensorManager = null;

	private Sensor
			sensor_Schritt = null, sensor_Schritt_Anzahl = null;

	private float[]
			Sensorwerte = null;

	private TextView
			text_sensor_schritt_wert = null, text_sensor_schritt_anzahl_wert = null;

	private ImageView
			image_sensor_gehen = null;

	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_sensor_schritt);                                           // Layout starten

		SchrittSensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);            // SensorManager anlegen
		LayoutInit();                                                                               // Textfelder initialisieren
	}

	private void LayoutInit()                                                                       // Variablen mit Steuerelementen füllen
	{
		sensor_Schritt = SchrittSensorManager.getDefaultSensor(Sensor.TYPE_STEP_DETECTOR);          // Schritterkennungssensor aktivieren
		sensor_Schritt_Anzahl = SchrittSensorManager.getDefaultSensor(Sensor.TYPE_STEP_COUNTER);    // Schrittanzahlsensor aktivieren

		text_sensor_schritt_wert = (TextView)findViewById(R.id.sensor_schritt_text_messwert);       // Textfelder initialisieren
		image_sensor_gehen = (ImageView)findViewById(R.id.sensor_schritt_image_maennchen);
		text_sensor_schritt_anzahl_wert = (TextView)findViewById(R.id.sensor_schritt_text_anzahl_wert);
	}

	public void onSensorChanged(SensorEvent event)                                                  // Wenn ein Sensorevent ausgelöst wird
	{
		if(event.sensor == sensor_Schritt)                                                          // War es der Schrittsensor?
		{
			Sensorwerte = null;
			Sensorwerte = event.values.clone();                                                     // Sensorwerte auslesen

			if(Sensorwerte[0] == 1)                                                                 // Wenn ein Schritt detektiert wurde
			{
				text_sensor_schritt_wert.setText(R.string.sensor_schritt_text_ja);                                            // Hinschreiben
				image_sensor_gehen.setImageResource(R.drawable.gehen);                              // Gehendes Männchen anzeigen

				Runnable mMyRunnable = new Runnable()
				{
					public void run()                                                               // Counter setzen, dass nach 500ms ohne neuen Schritt das Männchen und der Text zurückgesetzt werden
					{
						text_sensor_schritt_wert.setText(R.string.sensor_schritt_text_nein);                                  // Dann: Kein Schritt detektiert
						image_sensor_gehen.setImageResource(R.drawable.stehen);                     // Stehendes Männchen anzeigen
					}
				};

				Handler myHandler = new Handler();                                                  // Automatikhandler anlegen
				myHandler.postDelayed(mMyRunnable, 500);                                            // 500ms ohne erneuten Schritt -> dann alles wieder auf Anfang
			}
			else
			{
				text_sensor_schritt_wert.setText(R.string.sensor_schritt_text_nein);                                          // Wenn kein Schritt detektiert wird -> Dann auch hinschreiben
				image_sensor_gehen.setImageResource(R.drawable.stehen);                             // Stehendes Männchen
			}
		}
		if(event.sensor == sensor_Schritt_Anzahl)                                                   // Hat sich was an der Schrittanzahl geändert?
		{
			Sensorwerte = null;
			Sensorwerte = event.values.clone();                                                     // Anzahl auslesen

			text_sensor_schritt_anzahl_wert.setText(String.valueOf(Sensorwerte[0]));                // Anzahl ausgeben
		}
	}

	protected void onPause()                                                                        // Wenn App minimiert wird
	{
		super.onPause();
		SchrittSensorManager.unregisterListener(this);                                              // Schrittsensor deaktivieren
	}

	protected void onResume()                                                                       // Wenn App wieder maximiert wird
	{
		super.onResume();
		SchrittSensorManager.registerListener(this, sensor_Schritt,SensorManager.SENSOR_DELAY_NORMAL);  // Sensoren registrieren
		SchrittSensorManager.registerListener(this, sensor_Schritt_Anzahl,SensorManager.SENSOR_DELAY_NORMAL);   // Sensoren registrieren
	}

	public void onAccuracyChanged(Sensor sensor, int accuracy) {}

	public boolean onCreateOptionsMenu(Menu menu)                                                   // Menü anlegen
	{
		getMenuInflater().inflate(R.menu.menu_klein, menu);
		return true;
	}

	public boolean onOptionsItemSelected(MenuItem item)
	{
		switch (item.getItemId())
		{
			case R.id.action_info:
				Info();
				return true;

			default:
				return super.onOptionsItemSelected(item);
		}
	}

	private void Info()
	{
		AlertDialog alertDialog = new AlertDialog.Builder(this).create();
		alertDialog.setTitle("INFO");
		alertDialog.setMessage(getString(R.string.menue_info_text));
		alertDialog.setButton("OK", new DialogInterface.OnClickListener()
		{public void onClick(DialogInterface dialog, int which) {}});
		alertDialog.show();
	}
}
